#include "includeSdl.h"

#include "../../emu/h/types.h"
#include "../../nes/h/nesGeneral.h"
#include "../../nes/h/cpu.h"
#include "input.h"

char padOppositeLock = 0;

SDLKey padmode[4][8];				//4 mode, 8 buttons
u8 inputMode[2];

int numOfJoypads;
SDL_Joystick *jpad[4];

unsigned char *keyboardKeys;

struct tSdlPadHit
{
	s8		axisNum;
	s8		dpadNum;
	s8		buttonNum;
	Sint16	*pAxisTrigger;
	s8		*pAxis;
	s8		*pDpad;
	s8		*pButton;

	s8		*pButtonThisFrame;
	Sint16	*pAxisThisFrame;
	s8		*pDpadThisFrame;
};

tSdlPadHit padHit[4];

inline void padLock()
{
	if (padOppositeLock)
	{
		for (char n=0; n<2; n++)
		{
			if ((~pad[n] & 0x80) && (~pad[n] & 0x40))
				pad[n] |= 0x80;

			if ((~pad[n] & 0x20) && (~pad[n] & 0x10))
				pad[n] |= 0x20;
		}
	}
}

void joypadUpdate()
{
	int i, n;
	SDL_JoystickUpdate();
	for (i=0; i<numOfJoypads; i++)
	{
		for (n=0; n<padHit[i].axisNum; n++)
		{
			padHit[i].pAxisThisFrame[n]		= SDL_JoystickGetAxis(		jpad[i], n);
			n=n;
		}

		for (n=0; n<padHit[i].buttonNum; n++)
			padHit[i].pButtonThisFrame[n]	= SDL_JoystickGetButton(	jpad[i], n);

		for (n=0; n<padHit[i].dpadNum; n++)
			padHit[i].pDpadThisFrame[n]		= SDL_JoystickGetHat(		jpad[i], n);
	}
}

int joypadHit(int padNum, int buttonNum)
{
	int i;
	buttonNum = 1<<buttonNum;
	for (i=0; i<padHit[padNum].buttonNum; i++)
	{
		if ( (padHit[padNum].pButton[i] & buttonNum) && padHit[padNum].pButtonThisFrame[i]  )
			return 1;
	}
	
	for (i=0; i<padHit[padNum].dpadNum; i++)
	{
		if ( (padHit[padNum].pDpad[i*4 + 0] & buttonNum) && ( padHit[padNum].pDpadThisFrame[i] & SDL_HAT_UP ) )
			return 1;

		if ( (padHit[padNum].pDpad[i*4 + 1] & buttonNum) && ( padHit[padNum].pDpadThisFrame[i] & SDL_HAT_RIGHT ) )
			return 1;

		if ( (padHit[padNum].pDpad[i*4 + 2] & buttonNum) && ( padHit[padNum].pDpadThisFrame[i] & SDL_HAT_DOWN ) )
			return 1;

		if ( (padHit[padNum].pDpad[i*4 + 3] & buttonNum) && ( padHit[padNum].pDpadThisFrame[i] & SDL_HAT_LEFT ) )
			return 1;
	}

	for (i=0; i<padHit[padNum].axisNum; i++)
	{
		if ( (padHit[padNum].pAxis[i*2+0] & buttonNum)  && (padHit[padNum].pAxisTrigger[i*2 + 0] > padHit[padNum].pAxisThisFrame[i]) )
			return 1;

		if ( (padHit[padNum].pAxis[i*2+1] & buttonNum)  && (padHit[padNum].pAxisTrigger[i*2 + 1] < padHit[padNum].pAxisThisFrame[i]) )
			return 1;
	}
	
	return 0;
}

int keyboardHit(int padNum, int buttonNum)
{
	if (keyboardKeys == NULL)
		return 0;

	return keyboardKeys[padmode[inputMode[padNum]][7-buttonNum]];
}

void keyboardUpdate()
{
	keyboardKeys = SDL_GetKeyState(NULL);
}


void BuildKeys()
{
	joypadUpdate();
	keyboardUpdate();
	
	pad[0] = pad[1] = 0;
	for (int i=0; i<8; i++)
	{
		pad[0]<<=1;
		pad[0] |= ! ( keyboardHit(0, i) || joypadHit(0, i) );
		pad[1]<<=1;
		pad[1] |= ! ( keyboardHit(1, i) || joypadHit(1, i) );
	}	

	padLock();
}

void loadDefaultKeys()
{
	padmode[0][0] = SDLK_x;
	padmode[0][1] = SDLK_z;
	padmode[0][2] = SDLK_a;
	padmode[0][3] = SDLK_s;
	padmode[0][4] = SDLK_UP;
	padmode[0][5] = SDLK_DOWN;
	padmode[0][6] = SDLK_LEFT;
	padmode[0][7] = SDLK_RIGHT;
   
	padmode[1][0] = SDLK_PERIOD;
	padmode[1][1] = SDLK_KP0;
	padmode[1][2] = SDLK_q;
	padmode[1][3] = SDLK_KP_ENTER;
	padmode[1][4] = SDLK_w;
	padmode[1][5] = SDLK_s;
	padmode[1][6] = SDLK_a;
	padmode[1][7] = SDLK_d;

	padmode[2][0] = SDLK_s;
	padmode[2][1] = SDLK_a;
	padmode[2][2] = SDLK_LEFTBRACKET;
	padmode[2][3] = SDLK_RIGHTBRACKET;
	padmode[2][4] = SDLK_UP;
	padmode[2][5] = SDLK_DOWN;
	padmode[2][6] = SDLK_LEFT;
	padmode[2][7] = SDLK_RIGHT;

	padmode[3][0] = SDLK_COMMA;
	padmode[3][1] = SDLK_KP_PERIOD;
	padmode[3][2] = SDLK_k;
	padmode[3][3] = SDLK_l;
	padmode[3][4] = SDLK_KP8;
	padmode[3][5] = SDLK_KP5;
	padmode[3][6] = SDLK_KP4;
	padmode[3][7] = SDLK_KP6;

	inputMode[0] = 0;				//Why these keys and this mode?
	inputMode[1] = 3;				//cause i want it ;)
}

void uninitJoypad()
{
	int indx;
	indx = 0;
	while (indx < numOfJoypads)
	{
		if (jpad[indx] != 0)
		{
			delete padHit[indx].pAxis;
			delete padHit[indx].pAxisThisFrame;
			delete padHit[indx].pAxisTrigger;
			delete padHit[indx].pButton;
			delete padHit[indx].pButtonThisFrame;
			delete padHit[indx].pDpad;
			delete padHit[indx].pDpadThisFrame;
			SDL_JoystickClose( jpad[indx] );
			jpad[indx] = 0;
		}
		indx++;
	}
}

void defaultjoypadKeys(int indx)
{
	int i;
 	for (i=0; i < padHit[indx].buttonNum; i++)
		padHit[indx].pButton[i] = 0x80>>((i & 3) ^ 1 ^ ((i>>1) & 1) );

	for (i=0; i < padHit[indx].dpadNum; i++)
	{
		padHit[indx].pDpad[i*4 + 0] = 0x80>>nesUp;
		padHit[indx].pDpad[i*4 + 1] = 0x80>>nesRight;
		padHit[indx].pDpad[i*4 + 2] = 0x80>>nesDown;
		padHit[indx].pDpad[i*4 + 3] = 0x80>>nesLeft;
	}
//nesUp, nesDown, nesLeft, nesRight
	
	for (i=0; i < padHit[indx].axisNum; i++)
	{
		padHit[indx].pAxis[i*2 + 0] = 0x80>>(((i<<1) & 2) ^ ((i & 2)>>1) ^ 2+4);
		padHit[indx].pAxis[i*2 + 1] = 0x80>>(((i<<1) & 2) ^ ((i & 2)>>1) ^ 3+4);

		padHit[indx].pAxisTrigger[i*2 + 1] = 0.1 * 0x7FFF;
		padHit[indx].pAxisTrigger[i*2 + 0] = -padHit[indx].pAxisTrigger[i*2 + 1];
	}
}

void initJoypad()
{
	int padNum; const char *sz = SDL_JoystickName(0);
	numOfJoypads = SDL_NumJoysticks();
	for (padNum = 0; (padNum < numOfJoypads) && (padNum < 4); padNum++)
	{
		jpad[padNum] = SDL_JoystickOpen(padNum);
		padHit[padNum].axisNum			= SDL_JoystickNumAxes(		jpad[padNum] );
		padHit[padNum].buttonNum		= SDL_JoystickNumButtons(	jpad[padNum] );
		padHit[padNum].dpadNum			= SDL_JoystickNumHats(		jpad[padNum] );

		padHit[padNum].pAxisTrigger		= new Sint16[ padHit[padNum].axisNum * 2	];
		padHit[padNum].pAxis			= new s8	[ padHit[padNum].axisNum * 2	];
		padHit[padNum].pButton			= new s8	[ padHit[padNum].buttonNum		];
		padHit[padNum].pDpad			= new s8	[ padHit[padNum].dpadNum * 4	];

		padHit[padNum].pAxisThisFrame	= new Sint16[ padHit[padNum].axisNum		];
		padHit[padNum].pButtonThisFrame = new s8	[ padHit[padNum].buttonNum		];
		padHit[padNum].pDpadThisFrame	= new s8	[ padHit[padNum].dpadNum		];
		defaultjoypadKeys(padNum);
	}

	while (padNum < 4)
		jpad[padNum++] = 0;

}

void uninitInput()
{
	uninitJoypad();
}

int initInput()
{
	loadDefaultKeys();
	initJoypad();
	return 0;
}
